/*
** Copyright (c) 2002 D. Richard Hipp
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public
** License as published by the Free Software Foundation; either
** version 2 of the License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
** 
** You should have received a copy of the GNU General Public
** License along with this library; if not, write to the
** Free Software Foundation, Inc., 59 Temple Place - Suite 330,
** Boston, MA  02111-1307, USA.
**
** Author contact information:
**   drh@hwaci.com
**   http://www.hwaci.com/drh/
**
*******************************************************************************
**
** This file contains code used to generate convert plain text into
** similarly formated HTML.
*/
#include "config.h"
#include "format.h"
#include <time.h>

/*
** Output text while inserting hyperlinks to ticket and checkin reports.
** Within the text, an occurance of "#NNN" (where N is a digit) results
** in a hyperlink to the page that shows that ticket.  Any occurance of
** [NNN] gives a hyperlink to check-in number NNN.
*/
void output_with_inserted_links(const char *zText, int n){
  int i, j;
  for(i=0; i<n && zText[i]; i++){
    char *z;
    int c = zText[i];
    if( ((c!='#' || !g.okRead) && c!='[') || i+1>=n || !isdigit(zText[i+1]) ){
      continue;
    }
    for(j=2; i+j<n && isdigit(zText[i+j]); j++){}
    if( c=='[' && (i+j>=n || zText[i+j]!=']') ) continue;
    cgi_printf("%.*s", i, z);
    if( c=='#' ){
      cgi_printf("%.*s<a href=\"tktview?tn=%.*s\">%.*s</a>",
        i, zText, j-1, &zText[i+1], j, &zText[i]);
      zText += i+j;
      n -= i+j;
    }else{
      cgi_printf("%.*s<a href=\"chngview?cn=%.*s\">%.*s</a>",
        i, zText, j-1, &zText[i+1], j+1, &zText[i]);
      zText += i+j+1;
      n -= i+j+1;
    }
    i = -1;
  }
  if( i>0 ) cgi_printf("%.*s", i, zText);
}

/*
** z might be pointing to the beginning of a paragraph break.  Return the 
** number of characters in the paragraph break.  Return 0 if z is not
** pointing to a paragraph break.
**
** A paragraph break consists of one or more blank lines.
*/
static int paragraph_break_length(const char *z){
  int i;
  int n = 0;
  while( z && isspace(*z) ){
    i = 0;
    while( z[i]!='\n' && isspace(z[i]) ) i++;
    if( z[i]!='\n' ) break;
    n += i;
    z += i;
  }
  return n;
}

/*
** z is pointing to the beginning of a paragraph.  Return the number
** of characters in that paragraph.  If the return value is x then
** z[x] will point to either the start of the paragraph break or to
** the nul terminator at the end of z.
*/
static int paragraph_length(const char *z){
  
}


/*
** Output multi-line text as HTML.  Insert <p> in place of blank
** lines and <pre>...</pre> around text that is indented by two
** or more characters.
*/
void output_formatted(const char *zIn){
  int i = 0;
  int n;
  int inPre = 0;
  char *z, *zToFree;
  zIn += paragraph_break_length(zIn);
  z = zToFree = htmlize(zIn, -1);
  while( z && z[i] ){
    for(i=0; z[i] && z[i]!='\n'; i++){}
    output_with_inserted_links(z, i);
    z += i;
    if( z[0]==0 ) break;
    z++;
    for(i=n=0; isspace(z[i]); i++){
      if( z[i]=='\n' ) n++;
    }
    if( n==0 ){
      if( !inPre ) z += i;
      continue;
    }
    if( inPre ){ cgi_printf("</pre>\n"); inPre = 0; }
    inPre = i>=2 && z[i-1]!='\n' && z[i-2]!='\n';
    if( inPre ){
      cgi_printf("<pre>\n");
    }else{
      cgi_printf("<p>\n");
    }
    while( i>0 && z[i-1]!='\n' ){ i--; }
    z += i;
  }
  if( inPre ){ cgi_printf("</pre>\n"); }
  free(zToFree);
}
